/* Tabs */
QTabBar::tab {}
/* Active tab */
QTabBar::tab:selected {}
QComboBox {}
QLabel {}
QLineEdit {}
/* Group titles */
QLabel#settings_dialog_general_app_config_label,
QLabel#settings_dialog_general_app_main_menu_label,
QLabel#settings_dialog_general_statusbar_label,
QLabel#settings_dialog_general_app_font_size_label,
QLabel#settings_dialog_editor_config_label,
QLabel#settings_dialog_viewer_config_label,
QLabel#settings_dialog_ai_config_openai_api_label,
QLabel#settings_dialog_ai_config_base_label {
    color: gray;
}
QCheckBox {}
/* Custom slider (no ticks color supported) */
QSlider::groove:horizontal {}
QSlider::handle:horizontal {}
/* Request submit button */
QPushButton#settings_dialog_button_close {}
