# Russian lexemes ai_assistant.py
lexemes = {
    # ИИ помощник
    "dialog_title": "Запрос к ИИ",
    "dialog_prompt_input_placeholder_text": "Запрос",
    "dialog_prompt_input_accessible_description": "Введите запрос, чтобы дать контекст ИИ-помощнику",
    "dialog_response_output_placeholder_text": "Данные ответа",
    "dialog_response_output_accessible_description": "Данные ответа появятся в этом поле",
    "dialog_response_output_notice_empty_text": "Введите текст, чтобы получить ответ примерно...",

    "dialog_usage_model_label": "МОДЕЛЬ",
    "dialog_usage_tokens_label": "ТОКЕНЫ",
    "dialog_usage_tokens_prompt": "запрос: {tokens}",
    "dialog_usage_tokens_answer": "ответ: {tokens}",
    "dialog_usage_tokens_total": "всего: {tokens}",
    "dialog_button_send_request": "Отправить запрос",
}