# English lexemes ai_assistant.py
lexemes = {
    # AI assistant dialog
    "dialog_title": "Request to AI",
    "dialog_prompt_input_placeholder_text": "Request",
    "dialog_prompt_input_accessible_description": "Input request to give a context to the AI-assistant",
    "dialog_response_output_placeholder_text": "Response data",
    "dialog_response_output_accessible_description": "Response data will appear at this field",
    "dialog_response_output_notice_empty_text": "Provide a text input to get a response of about...",

    "dialog_usage_model_label": "MODEL",
    "dialog_usage_tokens_label": "TOKENS",
    "dialog_usage_tokens_prompt": "prompt: {tokens}",
    "dialog_usage_tokens_answer": "answer: {tokens}",
    "dialog_usage_tokens_total": "total: {tokens}",
    "dialog_button_send_request": "Send Request",
}