QToolBar > QCheckBox { /* QToolBar > QCheckBox: direct children of a QToolBar */
    margin-right: 0;
}
QToolBar QLineEdit#search_input {
    color: whitesmoke;
}
QToolBar QLabel#search_input_label {
    margin: 2px;
}
QToolBar QLabel#search_case_sensitive {
    margin: 0 2px;
}
