# Italian lexemes settings_dialog.py
lexemes = {
    # Impostazioni
    "window_title": "Impostazioni",

    "button_close": "Chiudi",

    "tab_general": "Generale",
    "tab_editor_config": "Editore",
    "tab_viewer_config": "Visualizzatore",
    "tab_ai_config": "Configurazione IA",

    "general_app_config_label": "Configurazione dell'app",
    "general_app_language_label": "Lingua",
    "general_app_language_combo_placeholder_text": "Scegli una lingua",
    "general_app_language_combo_accessible_description": "Lingua dell'interfaccia dell'app",
    "general_app_theme_label": "Tema",
    "general_app_theme_combo_placeholder_text": "Scegli un tema",
    "general_app_theme_combo_accessible_description": "Tema dell'interfaccia dell'app",
    "general_app_main_menu_label": "Menu Principale",
    "general_app_main_menu_checkbox": "Mostra il menu principale",
    "general_app_main_menu_checkbox_accessible_description": "Visualizza il menu a discesa principale dell'app",
    "general_app_font_size_label": "Dimensione del carattere: {size}pt",
    "general_app_font_size_slider_accessible_description": "Regola la dimensione del carattere globale dell'app",

    "general_statusbar_label": "Barra di Stato",
    "general_statusbar_show_global_cursor_position_checkbox": "Mostra la Posizione Globale del Cursore",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "Visualizza la posizione globale del cursore nella barra di stato",

    "editor_config_label": "Configurazione Editor",
    "editor_config_show_line_numbers_checkbox": "Mostra Numeri di Riga",
    "editor_config_show_line_numbers_checkbox_accessible_description": "Visualizza i numeri di riga nell'editor",

    "viewer_config_label": "Configurazione Visualizzatore",
    "viewer_config_process_emojis_checkbox": "Converti Emoji di Testo in Grafica",
    "viewer_config_process_emojis_checkbox_accessible_description":
        "Converti le emoji di testo in rappresentazioni grafiche",
    "viewer_config_highlight_todos_checkbox": "Evidenzia TODO",
    "viewer_config_highlight_todos_checkbox_accessible_description": "Evidenzia le etichette TODO nel testo",
    "viewer_config_open_link_confirmation_checkbox": "Richiedi Conferma per Aprire i Link",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "Richiedi conferma prima di aprire i link",
    "viewer_config_save_resources_checkbox": "Salva automaticamente le immagini esterne sul disco",
    "viewer_config_save_resources_checkbox_accessible_description":
        "Salva automaticamente copie delle immagini esterne sul disco per l'accesso offline.",

    "ai_config_openai_api_label": "API OpenAI",
    "ai_config_openai_api_url_input_placeholder_text": "URL API",
    "ai_config_openai_api_url_input_accessible_description": "URL dell'API OpenAI",
    "ai_config_openai_api_key_input_placeholder_text": "Chiave API",
    "ai_config_openai_api_key_input_accessible_description": "Chiave API di OpenAI",
    "ai_config_openai_api_supported_models_label": "Modelli supportati",
    "ai_config_ai_model_names_combo_placeholder_text": "Scegli modello",
    "ai_config_ai_model_names_combo_accessible_description": "Modelli supportati da scegliere",

    "ai_config_base_label": "Parametri Base",
    "ai_config_base_system_prompt_label": "Prompt del Sistema",
    "ai_config_base_system_prompt_edit_placeholder_text": "Prompt di sistema base che precede ogni richiesta",
    "ai_config_base_system_prompt_edit_accessible_description":
        "Prompt di sistema base che precede ogni richiesta. Testo semplice.",
    "ai_config_base_response_max_tokens_label": "Massimo di Token di Risposta",
    "ai_config_base_response_max_tokens_input_accessible_description": "Numero massimo di token da ricevere in risposta",
}