/* Request field */
QLineEdit {}
/* Response field */
QTextEdit {
    background-color: #fff7f7;
}
/* Token usage labels by id */
QLabel#ai_assistant_dialog_usage_model_label, QLabel#ai_assistant_dialog_usage_tokens_label {
    color: grey;
}
/* Request submit button */
QPushButton {
    background-color: crimson;
}