# Portuguese lexemes settings_dialog.py
lexemes = {
    # Configurações
    "window_title": "Configurações",

    "button_close": "Fechar",

    "tab_general": "Geral",
    "tab_editor_config": "Editor",
    "tab_viewer_config": "Visualizador",
    "tab_ai_config": "Configuração de IA",

    "general_app_config_label": "Configuração do aplicativo",
    "general_app_language_label": "Idioma",
    "general_app_language_combo_placeholder_text": "Escolher um idioma",
    "general_app_language_combo_accessible_description": "Idioma da interface do aplicativo",
    "general_app_theme_label": "Tema",
    "general_app_theme_combo_placeholder_text": "Escolher um tema",
    "general_app_theme_combo_accessible_description": "Tema da interface do aplicativo",
    "general_app_main_menu_label": "Menu Principal",
    "general_app_main_menu_checkbox": "Mostrar menu principal",
    "general_app_main_menu_checkbox_accessible_description": "Exibir o menu dropdown principal do aplicativo",
    "general_app_font_size_label": "Tamanho da fonte: {size}pt",
    "general_app_font_size_slider_accessible_description": "Ajustar o tamanho da fonte global do aplicativo",

    "general_statusbar_label": "Barra de Status",
    "general_statusbar_show_global_cursor_position_checkbox": "Mostrar Posição Global do Cursor",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "Exibir a posição global do cursor na barra de status",

    "editor_config_label": "Configuração do Editor",
    "editor_config_show_line_numbers_checkbox": "Mostrar Números de Linha",
    "editor_config_show_line_numbers_checkbox_accessible_description": "Exibir números de linha no editor",

    "viewer_config_label": "Configuração do Visualizador",
    "viewer_config_process_emojis_checkbox": "Converter Emojis de Texto em Gráficos",
    "viewer_config_process_emojis_checkbox_accessible_description":
        "Converter emojis de texto em representações gráficas",
    "viewer_config_highlight_todos_checkbox": "Destacar TODOs",
    "viewer_config_highlight_todos_checkbox_accessible_description": "Destacar tags TODO no texto",
    "viewer_config_open_link_confirmation_checkbox": "Requerer Confirmação para Abrir Links",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "Solicitar confirmação antes de abrir links",
    "viewer_config_save_resources_checkbox": "Salvar automaticamente imagens externas no disco",
    "viewer_config_save_resources_checkbox_accessible_description":
        "Salva automaticamente cópias de imagens externas no disco para acesso offline.",

    "ai_config_openai_api_label": "API da OpenAI",
    "ai_config_openai_api_url_input_placeholder_text": "URL da API",
    "ai_config_openai_api_url_input_accessible_description": "URL da API da OpenAI",
    "ai_config_openai_api_key_input_placeholder_text": "Chave da API",
    "ai_config_openai_api_key_input_accessible_description": "Chave da API da OpenAI",
    "ai_config_openai_api_supported_models_label": "Modelos suportados",
    "ai_config_ai_model_names_combo_placeholder_text": "Escolher modelo",
    "ai_config_ai_model_names_combo_accessible_description": "Modelos suportados para escolha",

    "ai_config_base_label": "Parâmetros Base",
    "ai_config_base_system_prompt_label": "Prompt do Sistema",
    "ai_config_base_system_prompt_edit_placeholder_text": "Prompt do sistema base que antecede cada solicitação",
    "ai_config_base_system_prompt_edit_accessible_description":
        "Prompt do sistema base que antecede cada solicitação. Texto simples.",
    "ai_config_base_response_max_tokens_label": "Máximo de Tokens de Resposta",
    "ai_config_base_response_max_tokens_input_accessible_description": "Número máximo de tokens a receber em resposta",
}